#!/usr/bin/env python3

import ctypes
from ctypes import *
from enum import Enum
import os
import syslog
from pydbus import SystemBus
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, Gtk, Gio

readonly_list = ["/etc/mdm/storage", "/etc/mdm/white-list", "/etc/mdm/black-list"]
# readonly_list = ["/home/xc/1.txt", "/home/xc/2.txt", "/home/xc/3.txt"]

class kysec_modular(Enum):
    KYSEC_EXECTL = 0
    KYSEC_NETCTL = 1
    KYSEC_FPRO = 2
    KYSEC_KMOD = 3
    KYSEC_3ADM = 4
    KYSEC_PPRO = 5
    KYSEC_DEVCTL = 6
    KYSEC_IPT = 7
    KYSEC_MAX = 8

class kysec_func_status(Enum):
    FUNC_DISABLED = 0
    FUNC_ENABLED = 1
    FUNC_WARNING = 2


class KdkConfReinforce:
    """
    <node>
        <interface name='com.kylin.kysdk.confReinforce'>
            <method name='set_conf_reinforce_Status' >
                <arg type='i' direction='in' name='status'/>
                <arg type='i' direction='out' name='rc'/>
            </method>
            <method name='get_conf_reinforce_Status' >
                <arg type='i' direction='out' name='status'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        self.file_perm = "/etc/kysdk/kysdk-security/conf-reinforce.perm"
        try:
            self.extend_so = CDLL("libkysec_extend.so.0")
        except Exception:
            exit(-1)

    def set_conf_reinforce_Status(self, status):
        """
        status
            0 开启文件防篡改（存在标记文件）
            1 关闭文件防篡改（不存在标记文件）
        
        """
        if status == 0:
            # 创建文件
            with open(self.file_perm, 'w') as f:
                f.write('0')
            # 设置文件防篡改
            for filepath in readonly_list:
                self.check_file_exist(filepath)
                rc = self.set_file_readonly(filepath)
                if rc != 0:
                    syslog.syslog(syslog.LOG_DEBUG, "set_file_readonly failed")
                    return -1
        elif status == 1:
            #  删除文件
            if os.path.exists(self.file_perm):
                os.remove(self.file_perm)
            # 取消设置文件防篡改
            for filepath in readonly_list:
                self.check_file_exist(filepath)
                rc = self.del_file_readonly(filepath)
                if rc != 0:
                    syslog.syslog(syslog.LOG_DEBUG, "del_file_readonly failed")
                    return -1
        else:
            return -1
        
        return 0

    def get_conf_reinforce_Status(self):
        if os.path.exists(self.file_perm):
            return 0
        else:
            return 1

    def check_file_exist(self, filepath):
        if not os.path.exists(filepath):
            with open(filepath, 'w') as f:
                syslog.syslog(syslog.LOG_DEBUG, "create file {}".format(filepath))
        return 0

    def set_file_readonly(self, filepath):
        rc = self.kysec_fpro_enable()
        if rc != 0:
            syslog.syslog(syslog.LOG_DEBUG, "enable kysec process protect failed")
            return rc

        rc = self.extend_so.kysec_protect_set(ctypes.c_char_p(bytes(filepath, encoding = "utf8")), ctypes.c_char_p(bytes("readonly", encoding = "utf8")));
        if rc != 0:
            syslog.syslog(syslog.LOG_DEBUG, "set kysec protect readonly failed")
            return rc

        return 0

    def del_file_readonly(self, filepath):
        rc = self.kysec_fpro_enable()
        if rc != 0:
            syslog.syslog(syslog.LOG_DEBUG, "enable kysec process protect failed")
            return rc

        rc = self.extend_so.kysec_protect_set(ctypes.c_char_p(bytes(filepath, encoding = "utf8")), ctypes.c_char_p(bytes("none", encoding = "utf8")));
        if rc != 0:
            syslog.syslog(syslog.LOG_DEBUG, "cancel set kysec protect readonly failed")
            return rc

        return 0

    def kysec_fpro_enable(self):
        rc = self.extend_so.kysec_getstatus()
        if rc != 2:
            syslog.syslog(syslog.LOG_DEBUG, "kysec get status failed")
            return -1

        rc = self.extend_so.kysec_get_func_status(c_int(kysec_modular.KYSEC_FPRO.value))
        if rc == 1 or rc == 2:
            return 0

        if rc == 0:
            rc = self.extend_so.kysec_set_func_status(c_int(kysec_modular.KYSEC_FPRO.value), c_int(kysec_func_status.FUNC_ENABLED.value))
            if rc != 0:
                syslog.syslog(syslog.LOG_DEBUG, "kysec set fpro func status failed")
                return -1

            rc = self.extend_so.kysec_conf_set("kysec_fpro", c_int(kysec_func_status.FUNC_ENABLED.value))
            if rc != 0:
                syslog.syslog(syslog.LOG_DEBUG, "kysec set conf failed")
                return -1

        return 0

if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.kysdk.confReinforce", KdkConfReinforce())
    loop.run()
# controler = KdkConfReinforce()
# rc = controler.set_conf_reinforce_Status(0)
# print(rc)
# rc = controler.get_conf_reinforce_Status()
# print(rc)


